@@ -27,28 +27,30 @@
 #       (currently cleaned up so switched off)
 ##############################################################################
 
+import copy as _copy
+import sys as _sys
+import warnings as _warnings
+from collections import OrderedDict as _OrderedDict
+
+import numpy as _np
+
 # Required dependencies (private)
 import xarray as _xr
-import copy as _copy
-import numpy as _np
-import warnings as _warnings
-import sys as _sys
-from collections import OrderedDict as _OrderedDict
 
 # From OceanSpy (private)
 from . import utils as _utils
 from ._ospy_utils import (
     _check_instance,
-    _check_oceanspy_axes,
-    _setter_error_message,
     _check_list_of_string,
+    _check_oceanspy_axes,
     _create_grid,
     _rename_coord_attrs,
+    _setter_error_message,
 )
-from .subsample import _subsampleMethods
+from .animate import _animateMethods
 from .compute import _computeMethods
 from .plot import _plotMethods
-from .animate import _animateMethods
+from .subsample import _subsampleMethods
 
 # Recommended dependencies (private)
 try:
@@ -235,6 +237,9 @@ class OceanDataset:
 
         return self
 
+    def __getitem__(self, key):
+        return self._ds[key]
+
     # -------------------
     # aliases
     # -------------------
@@ -393,7 +398,7 @@ class OceanDataset:
         parameters: dict
             {'name': value}
         """
-        from oceanspy import DEFAULT_PARAMETERS, AVAILABLE_PARAMETERS, TYPE_PARAMETERS
+        from oceanspy import AVAILABLE_PARAMETERS, DEFAULT_PARAMETERS, TYPE_PARAMETERS
 
         # Check parameters
         _check_instance({"parameters": parameters}, "dict")
@@ -622,16 +627,20 @@ class OceanDataset:
         # check parameters
         _check_instance({"face_connections": face_connections}, "dict")
 
-        for k in face_connections["face"].keys():
-            for axis in face_connections["face"][k].keys():
-                if type(face_connections["face"][k][axis]) == tuple:
-                    face_connections["face"][k][axis] = face_connections["face"][k][
-                        axis
-                    ]
-                else:
-                    face_connections["face"][k][axis] = eval(
-                        face_connections["face"][k][axis]
-                    )
+        if list(face_connections)[0] == "face":
+
+            for k in face_connections["face"].keys():
+                for axis in face_connections["face"][k].keys():
+                    if type(face_connections["face"][k][axis]) == tuple:
+                        face_connections["face"][k][axis] = face_connections["face"][k][
+                            axis
+                        ]
+                    else:
+                        face_connections["face"][k][axis] = eval(
+                            face_connections["face"][k][axis]
+                        )
+        elif list(face_connections)[0] is None:
+            face_connections = None
 
         self = self._store_as_global_attr(
             name="face_connections", attr=face_connections, overwrite=True
@@ -824,9 +833,10 @@ class OceanDataset:
             z = _xr.zeros_like(Y)
 
         # Stack
-        x_stack = x.stack(points=x.dims).values
-        y_stack = y.stack(points=y.dims).values
-        z_stack = z.stack(points=z.dims).values
+        rid_value = 777777
+        x_stack = x.stack(points=x.dims).fillna(rid_value).data
+        y_stack = y.stack(points=y.dims).fillna(rid_value).data
+        z_stack = z.stack(points=z.dims).fillna(rid_value).data
 
         # Construct KD-tree
         tree = _spatial.cKDTree(_np.column_stack((x_stack, y_stack, z_stack)))
