@@ -20,22 +20,25 @@ Create new variables using OceanDataset objects.
 # TODO: compute velocity magnitude
 #############################################################
 
-# Required dependencies (private)
-import xarray as _xr
-import oceanspy as _ospy
-import numpy as _np
-import warnings as _warnings
 import copy as _copy
 import functools as _functools
+import warnings as _warnings
 from collections import OrderedDict as _OrderedDict
 
+import numpy as _np
+
+# Required dependencies (private)
+import xarray as _xr
+
+import oceanspy as _ospy
+
 # From OceanSpy (private)
 from . import utils as _utils
 from ._ospy_utils import (
+    _check_ijk_components,
     _check_instance,
     _check_list_of_string,
     _handle_aliased,
-    _check_ijk_components,
     _rename_aliased,
 )
 
@@ -86,6 +89,7 @@ _FUNC2VARS = _OrderedDict(
     geographical_aligned_velocities=["U_zonal", "V_merid"],
     survey_aligned_velocities=["rot_ang_Vel", "tan_Vel", "ort_Vel"],
     missing_horizontal_spacing=["dxF", "dxV", "dyF", "dyU"],
+    missing_cs_sn=["CS", "SN"],
 )
 
 
@@ -2080,6 +2084,8 @@ def mooring_volume_transport(od):
     mooring = od._ds["mooring"]
     XC = od._ds["XC"].squeeze(("Y", "X"))
     YC = od._ds["YC"].squeeze(("Y", "X"))
+    Xind = od._ds["Xind"].squeeze(("Y", "X"))
+    Yind = od._ds["Yind"].squeeze(("Y", "X"))
     XU = od._ds["XU"].squeeze(("Y"))
     YU = od._ds["YU"].squeeze(("Y"))
     XV = od._ds["XV"].squeeze(("X"))
@@ -2094,10 +2100,10 @@ def mooring_volume_transport(od):
     )
 
     # Extract left and right values
-    U1 = U_tran.isel(Xp1=1)
-    U0 = U_tran.isel(Xp1=0)
-    V1 = V_tran.isel(Yp1=1)
-    V0 = V_tran.isel(Yp1=0)
+    U1 = U_tran.isel(Xp1=1).fillna(0)
+    U0 = U_tran.isel(Xp1=0).fillna(0)
+    V1 = V_tran.isel(Yp1=1).fillna(0)
+    V0 = V_tran.isel(Yp1=0).fillna(0)
 
     # Initialize direction
     U0_dir = _np.zeros((len(XC), 2))
@@ -2106,8 +2112,8 @@ def mooring_volume_transport(od):
     V1_dir = _np.zeros((len(YC), 2))
 
     # Steps
-    diffX = _np.diff(XC)
-    diffY = _np.diff(YC)
+    diffX = _np.diff(Xind)
+    diffY = _np.diff(Yind)
 
     # Closed array?
     if XC[0] == XC[-1] and YC[0] == YC[-1]:
@@ -2828,6 +2834,36 @@ def salt_budget(od):
     return _ospy.OceanDataset(ds).dataset
 
 
+def missing_cs_sn(od):
+    """
+    Compute missing grid orientaioon.
+
+    Parameters
+    ----------
+    od: OceanDataset
+        oceandataset used to compute
+
+    Returns
+    -------
+    od: OceanDataset
+        | CS: cosine of the orientation angle
+        | SN: sine of the orientation angle
+    """
+    xc = _np.deg2rad(_np.array(od._ds.XC))
+    yc = _np.deg2rad(_np.array(od._ds.YC))
+    cs = _np.zeros_like(xc)
+    sn = _np.zeros_like(xc)
+    cs[0], sn[0] = _utils.find_cs_sn(yc[0], xc[0], yc[1], xc[1])
+    cs[-1], sn[-1] = _utils.find_cs_sn(yc[-2], xc[-2], yc[-1], xc[-1])
+    cs[1:-1], sn[1:-1] = _utils.find_cs_sn(yc[:-2], xc[:-2], yc[2:], xc[2:])
+    od._ds["CS"] = od._ds["XC"]
+    od._ds["CS"].values = cs
+
+    od._ds["SN"] = od._ds["XC"]
+    od._ds["SN"].values = sn
+    return od
+
+
 def missing_horizontal_spacing(od):
     """
     Compute missing horizontal spacing.
