@@ -11,26 +11,30 @@ Subsample OceanDataset objects.
 # 5. Add new functions to _subsampleMethods
 # 6. Add new functions to docs/api.rst
 
+import copy as _copy
+import functools as _functools
+import warnings as _warnings
+
+import numpy as _np
+import pandas as _pd
+
 # Required dependencies (private)
 import xarray as _xr
-import pandas as _pd
-import numpy as _np
-import copy as _copy
-import warnings as _warnings
-import functools as _functools
+from packaging.version import parse as _parse_version
 
 # From OceanSpy (private)
-from . import utils as _utils
 from . import compute as _compute
-from .llc_rearrange import LLCtransformation as _llc_trans
+from . import utils as _utils
 from ._ospy_utils import (
     _check_instance,
-    _check_range,
     _check_list_of_string,
     _check_native_grid,
     _check_part_position,
+    _check_range,
     _rename_aliased,
 )
+from .llc_rearrange import LLCtransformation as _llc_trans
+from .utils import _rel_lon, _reset_range, get_maskH
 
 # Recommended dependencies (private)
 try:
@@ -56,8 +60,8 @@ def cutout(
     timeFreq=None,
     sampMethod="snapshot",
     dropAxes=False,
-    transformation=False,
-    centered="Atlantic",
+    centered=None,
+    chunks=None,
 ):
     """
     Cutout the original dataset in space and time
@@ -104,13 +108,10 @@ def cutout(
         if one point only is in the range.
         If True, set dropAxes=od.grid_coords.
         If False, preserve original grid.
-    transformation: str, or bool
-        Lists the transformation of the llcgrid into a new one in which face
-        is no longer a dimension. Default is `False`. If `True`, need to
-        define how data will be centered
-    centered: str, or bool
-        default is `Atlantic`, and other options is `Pacific`. This refers
-        to which ocean appears centered on the data.
+    centered: str or bool.
+        Only used when `face` is a dimension. When str, 'Atlantic' or 'Pacific'
+        are the only possible choices. Default is `None` and centered is estimated
+        during the cutout.
 
     Returns
     -------
@@ -334,7 +335,7 @@ def cutout(
                 _np.fabs(od._ds["YG"].max() - od._ds["YG"].min()),
             ]
         )
-        add_Hbdr = add_Hbdr / _np.mean([len(od._ds["X"]), len(od._ds["Y"])])
+        add_Hbdr = 1.5 * add_Hbdr / _np.mean([len(od._ds["X"]), len(od._ds["Y"])])
     elif add_Hbdr is False:
         add_Hbdr = 0
 
@@ -343,60 +344,51 @@ def cutout(
     elif add_Vbdr is False:
         add_Vbdr = 0
 
-    # Initialize horizontal mask
-    if XRange is not None or YRange is not None:
+    if "face" in ds.dims:
 
-        maskH, dmaskH, XRange, YRange = get_maskH(
-            ds, add_Hbdr, add_Vbdr, XRange, YRange
-        )
-
-    if transformation is not False and "face" in ds.dims:
-        if XRange is None and YRange is None:
-            faces = "all"
-        else:
-            faces = dmaskH["face"].values  # gets faces that survives cutout
-        _transf_list = ["arctic_crown", "arctic_centered"]
-        if transformation in _transf_list:
-            arg = {
-                "ds": ds,
-                "varlist": varList,  # vars and grid coords to transform
-                "centered": centered,
-                "faces": faces,
-                "drop": True,  # required to calculate U-V grid points
-            }
-            if transformation == "arctic_crown":
-                _transformation = _llc_trans.arctic_crown
-            elif transformation == "arctic_centered":
-                _transformation = _llc_trans.arctic_centered
-            dsnew = _transformation(**arg)
-            dsnew = dsnew.set_coords(co_list)
-            grid_coords = od.grid_coords
-            od._ds = dsnew
-            manipulate_coords = {"coordsUVfromG": True}
-            od = od.manipulate_coords(**manipulate_coords)
-            if len(grid_coords["time"]) > 1:
-                grid_coords["time"].pop("time_midp", None)
-                grid_coords = {"add_midp": True, "grid_coords": grid_coords}
-            od = od.set_grid_coords(**grid_coords, overwrite=True)
-            od._ds.attrs["OceanSpy_description"] = "Cutout of"
-            "simulation, with simple topology (face not a dimension)"
-            # Unpack the new dataset without face as dimension
-            ds = od._ds
-            maskH, dmaskH, XRange, YRange = get_maskH(
-                ds, add_Hbdr, add_Vbdr, XRange, YRange
-            )
-        elif transformation not in _transf_list:
-            raise ValueError("transformation not supported")
-    elif transformation is False and "face" in ds.dims:
-        raise ValueError(
-            "Must define a transformation to remove complex" "topology of dataset."
-        )
+        arg = {
+            "ds": ds,
+            "varList": varList,  # vars and grid coords to transform
+            "add_Hbdr": add_Hbdr,
+            "XRange": XRange,
+            "YRange": YRange,
+            "centered": centered,
+            "drop": True,  # required to calculate U-V grid points
+            "chunks": chunks,
+        }
+        dsnew = _llc_trans.arctic_crown(**arg)
+        dsnew = dsnew.set_coords(co_list)
+        # grid_coords = od.grid_coords
+        grid_coords = {
+            "Y": {"Y": None, "Yp1": 0.5},
+            "X": {"X": None, "Xp1": 0.5},
+            "Z": {"Z": None, "Zp1": 0.5, "Zu": 0.5, "Zl": -0.5},
+            "time": {"time": -0.5},
+        }
+        # if len(grid_coords["time"]) > 1:
+        # grid_coords["time"].pop("time_midp", None)
+        grid_coords = {"add_midp": True, "overwrite": True, "grid_coords": grid_coords}
+        od._ds = dsnew
+        manipulate_coords = {"coordsUVfromG": True}
+        new_face_connections = {"face_connections": {None: {None, None}}}
+        od = od.set_face_connections(**new_face_connections)
+        od = od.manipulate_coords(**manipulate_coords)
+        od = od.set_grid_coords(**grid_coords)
+        od._ds.attrs["OceanSpy_description"] = "Cutout of"
+        "simulation, with simple topology (face not a dimension)"
+        # Unpack the new dataset without face as dimension
+        ds = od._ds
 
     # ---------------------------
     # Horizontal CUTOUT part II (continuation of original code)
     # ---------------------------
-
+    # Initialize horizontal mask
     if XRange is not None or YRange is not None:
+        XRange, ref_lon = _reset_range(XRange)
+        maskH, dmaskH, XRange, YRange = get_maskH(
+            ds, add_Hbdr, XRange, YRange, ref_lon=ref_lon
+        )
+
         dYp1 = dmaskH["Yp1"].values
         dXp1 = dmaskH["Xp1"].values
         iY = [_np.min(dYp1), _np.max(dYp1)]
@@ -482,7 +474,10 @@ def cutout(
         maskC = _xr.where(
             _np.logical_and(
                 _np.logical_and(ds["YC"] >= minY, ds["YC"] <= maxY),
-                _np.logical_and(ds["XC"] >= minX, ds["XC"] <= maxX),
+                _np.logical_and(
+                    _rel_lon(ds["XC"], ref_lon) >= _rel_lon(minX, ref_lon),
+                    _rel_lon(ds["XC"], ref_lon) <= _rel_lon(maxX, ref_lon),
+                ),
             ),
             1,
             0,
@@ -490,7 +485,10 @@ def cutout(
         maskG = _xr.where(
             _np.logical_and(
                 _np.logical_and(ds["YG"] >= minY, ds["YG"] <= maxY),
-                _np.logical_and(ds["XG"] >= minX, ds["XG"] <= maxX),
+                _np.logical_and(
+                    _rel_lon(ds["XG"], ref_lon) >= _rel_lon(minX, ref_lon),
+                    _rel_lon(ds["XG"], ref_lon) <= _rel_lon(maxX, ref_lon),
+                ),
             ),
             1,
             0,
@@ -498,7 +496,10 @@ def cutout(
         maskU = _xr.where(
             _np.logical_and(
                 _np.logical_and(ds["YU"] >= minY, ds["YU"] <= maxY),
-                _np.logical_and(ds["XU"] >= minX, ds["XU"] <= maxX),
+                _np.logical_and(
+                    _rel_lon(ds["XU"], ref_lon) >= _rel_lon(minX, ref_lon),
+                    _rel_lon(ds["XU"], ref_lon) <= _rel_lon(maxX, ref_lon),
+                ),
             ),
             1,
             0,
@@ -506,7 +507,10 @@ def cutout(
         maskV = _xr.where(
             _np.logical_and(
                 _np.logical_and(ds["YV"] >= minY, ds["YV"] <= maxY),
-                _np.logical_and(ds["XV"] >= minX, ds["XV"] <= maxX),
+                _np.logical_and(
+                    _rel_lon(ds["XV"], ref_lon) >= _rel_lon(minX, ref_lon),
+                    _rel_lon(ds["XV"], ref_lon) <= _rel_lon(maxX, ref_lon),
+                ),
             ),
             1,
             0,
@@ -650,6 +654,12 @@ def mooring_array(od, Ymoor, Xmoor, **kwargs):
         Subsampled oceandataset.
     """
 
+    # Add indexes needed for transports
+    Yind, Xind = _xr.broadcast(od._ds["Y"], od._ds["X"])
+    od._ds["Xind"] = Xind.transpose(*od._ds["XC"].dims)
+    od._ds["Yind"] = Yind.transpose(*od._ds["YC"].dims)
+    od._ds = od._ds.set_coords(["Xind", "Yind"])
+
     # Check
     _check_native_grid(od, "mooring_array")
 
@@ -1086,7 +1096,15 @@ def survey_stations(
     )
 
     # Interpolate
-    regridder = _xe.Regridder(ds_in, ds, **xesmf_regridder_kwargs)
+    try:
+        regridder = _xe.Regridder(ds_in, ds, **xesmf_regridder_kwargs)
+    except ValueError:
+        raise ValueError(
+            """
+        An error occured when creating the xesmf.Regridder object,
+        try add_Hbdr = M, where M>1.5 times horizontal spacing
+        """
+        )
     regridder._grid_in = None  # See https://github.com/JiaweiZhuang/xESMF/issues/71
     regridder._grid_out = None  # See https://github.com/JiaweiZhuang/xESMF/issues/71
     interp_vars = [
@@ -1106,7 +1124,8 @@ def survey_stations(
             ds[var].attrs = attrs
         elif var not in ["Xp1", "Yp1"]:
             ds[var] = ds_in[var].reset_coords(drop=True)
-    regridder.clean_weight_file()
+    if _parse_version(_xe.__version__) < _parse_version("0.4.0"):
+        regridder.clean_weight_file()
 
     # Extract transect
     ds = ds.isel(
@@ -1336,52 +1355,6 @@ def particle_properties(od, times, Ypart, Xpart, Zpart, **kwargs):
     return od
 
 
-def get_maskH(ds, add_Hbdr, add_Vbdr, XRange, YRange):
-    """ Define this function to avoid repeated code. First time this runs,
-    the objective is to figure out which faces survive the cutout. This info
-    is then passed, when transforming llc-grids, to llc_rearrange. Second
-    time this code runs, it gets applied on a dataset without faces as a
-    dimension.
-    """
-    maskH = _xr.ones_like(ds["XG"])
-
-    if YRange is not None:
-        # Use arrays
-        YRange = _np.asarray([_np.min(YRange) - add_Hbdr, _np.max(YRange) + add_Hbdr])
-        YRange = YRange.astype(ds["YG"].dtype)
-
-        # Get the closest
-        for i, Y in enumerate(YRange):
-            diff = _np.fabs(ds["YG"] - Y)
-            YRange[i] = ds["YG"].where(diff == diff.min()).min().values
-        maskH = maskH.where(
-            _np.logical_and(ds["YG"] >= YRange[0], ds["YG"] <= YRange[-1]), 0
-        )
-
-    if XRange is not None:
-        # Use arrays
-        XRange = _np.asarray([_np.min(XRange) - add_Hbdr, _np.max(XRange) + add_Hbdr])
-        XRange = XRange.astype(ds["XG"].dtype)
-
-        # Get the closest
-        for i, X in enumerate(XRange):
-            diff = _np.fabs(ds["XG"] - X)
-            XRange[i] = ds["XG"].where(diff == diff.min()).min().values
-        maskH = maskH.where(
-            _np.logical_and(ds["XG"] >= XRange[0], ds["XG"] <= XRange[-1]), 0
-        )
-    # Can't be all zeros
-    if maskH.sum() == 0:
-        raise ValueError("Zero grid points in the horizontal range")
-
-    # Find horizontal indexes
-    maskH = maskH.assign_coords(
-        Yp1=_np.arange(len(maskH["Yp1"])), Xp1=_np.arange(len(maskH["Xp1"]))
-    )
-    dmaskH = maskH.where(maskH, drop=True)
-    return maskH, dmaskH, XRange, YRange
-
-
 class _subsampleMethods(object):
     """
     Enables use of functions as OceanDataset attributes.
