@@ -1,36 +1,37 @@
 # Import modules
-import pytest
 import numpy as np
+import pytest
 import xarray as xr
+from numpy.testing import assert_allclose, assert_almost_equal, assert_array_equal
+
 import oceanspy as ospy
 
 # From OceanSpy
-from oceanspy import open_oceandataset
-from oceanspy import AVAILABLE_PARAMETERS
-from oceanspy.compute import gradient, divergence, curl
+from oceanspy import AVAILABLE_PARAMETERS, open_oceandataset
 from oceanspy.compute import (
-    missing_horizontal_spacing,
-    potential_density_anomaly,
     Brunt_Vaisala_frequency,
-    velocity_magnitude,
-    horizontal_velocity_magnitude,
-    vertical_relative_vorticity,
-    relative_vorticity,
-    kinetic_energy,
-    eddy_kinetic_energy,
-    horizontal_divergence_velocity,
-    shear_strain,
-    normal_strain,
-    Okubo_Weiss_parameter,
     Ertel_potential_vorticity,
-    mooring_volume_transport,
-    survey_aligned_velocities,
+    Okubo_Weiss_parameter,
+    curl,
+    divergence,
+    eddy_kinetic_energy,
+    gradient,
     heat_budget,
+    horizontal_divergence_velocity,
+    horizontal_velocity_magnitude,
+    kinetic_energy,
+    missing_horizontal_spacing,
+    mooring_volume_transport,
+    normal_strain,
+    potential_density_anomaly,
+    relative_vorticity,
     salt_budget,
+    shear_strain,
+    survey_aligned_velocities,
+    velocity_magnitude,
+    vertical_relative_vorticity,
 )
 
-from numpy.testing import assert_allclose, assert_array_equal, assert_almost_equal
-
 # Directory
 Datadir = "./oceanspy/tests/Data/"
 
@@ -127,7 +128,7 @@ def test_velocity_magnitude(od_in):
     U = (od_in._ds["U"].values[:, :, :, 1:] + od_in._ds["U"].values[:, :, :, :-1]) / 2
     V = (od_in._ds["V"].values[:, :, 1:, :] + od_in._ds["V"].values[:, :, :-1, :]) / 2
     W = (od_in._ds["W"].values[:, 1:, :, :] + od_in._ds["W"].values[:, :-1, :, :]) / 2
-    vel = np.sqrt(U[:, :-1, :, :] ** 2 + V[:, :-1, :, :] ** 2 + W ** 2)
+    vel = np.sqrt(U[:, :-1, :, :] ** 2 + V[:, :-1, :, :] ** 2 + W**2)
     assert_allclose(vel, ds_out["vel"].values[:, :-1, :, :])
 
     # Test shortcut
@@ -146,7 +147,7 @@ def test_horizontal_velocity_magnitude(od_in):
     # Check values
     U = (od_in._ds["U"].values[:, :, :, 1:] + od_in._ds["U"].values[:, :, :, :-1]) / 2
     V = (od_in._ds["V"].values[:, :, 1:, :] + od_in._ds["V"].values[:, :, :-1, :]) / 2
-    hor_vel = np.sqrt(U ** 2 + V ** 2)
+    hor_vel = np.sqrt(U**2 + V**2)
     assert_allclose(hor_vel, ds_out["hor_vel"].values)
 
     # Test shortcut
@@ -245,7 +246,7 @@ def test_kinetic_energy(od_in, eps_nh):
     # Check values
     U = (od_in._ds["U"].values[:, :, :, 1:] + od_in._ds["U"].values[:, :, :, :-1]) / 2
     V = (od_in._ds["V"].values[:, :, 1:, :] + od_in._ds["V"].values[:, :, :-1, :]) / 2
-    KE = (U ** 2 + V ** 2) / 2
+    KE = (U**2 + V**2) / 2
     if eps_nh == 0:
         assert_allclose(KE, ds_out["KE"].values)
     else:
@@ -275,7 +276,7 @@ def test_eddy_kinetic_energy(od_in, eps_nh):
     V = od_in._ds["V"] - od_in._ds["V"].mean("time")
     U = (U.values[:, :, :, 1:] + U.values[:, :, :, :-1]) / 2
     V = (V.values[:, :, 1:, :] + V.values[:, :, :-1, :]) / 2
-    EKE = (U ** 2 + V ** 2) / 2
+    EKE = (U**2 + V**2) / 2
     if eps_nh == 0:
         assert_allclose(EKE, ds_out["EKE"].values)
     else:
@@ -469,14 +470,14 @@ def test_survey_aligned_velocities(od_in, gridtype, rotate, survey):
             # Test shortcut
             od_surv.compute.survey_aligned_velocities()
 
-        # Chek velocities
+        # Check velocities
         vel_surv = np.sqrt(od_surv._ds["U"] ** 2 + od_surv._ds["V"] ** 2)
         vel_alig = np.sqrt(ds_out["tan_Vel"] ** 2 + ds_out["ort_Vel"] ** 2)
-        if rotate is False:
-            assert_allclose(vel_surv.values, vel_alig.values)
-        else:
-            # TODO: not exact match. Check with Renske!
-            assert_almost_equal(vel_surv.values, vel_alig.values, 4)
+        # see issue 70 for more information.
+        # https://github.com/hainegroup/oceanspy/issues/70
+        assert_almost_equal(
+            vel_surv.values, vel_alig.values, decimal=4 if rotate else 7
+        )
 
     else:
         with pytest.raises(ValueError):
