@@ -1,11 +1,11 @@
 # TODO: cartesian, and Xp1 Yp1 right are not tested.
-import pytest
 import numpy as np
+import pytest
 import xarray as xr
 from numpy.testing import assert_array_equal
 
 # From OceanSpy
-from oceanspy import open_oceandataset, OceanDataset
+from oceanspy import OceanDataset, open_oceandataset
 
 # Directory
 Datadir = "./oceanspy/tests/Data/"
@@ -261,36 +261,65 @@ def test_reduce_variables(od, varList):
 
 @pytest.mark.parametrize("od", [ECCOod])
 @pytest.mark.parametrize(
-    "XRange, YRange, ZRange, varList, transformation, centered, NZ",
+    "XRange, YRange, ZRange, varList, NZ, NY, NX",
     [
-        ([-90, 20], [20, 80], None, ["T"], "arctic_crown", "Atlantic", 50),
-        ([-179, 179], [20, 80], None, ["U", "V"], "arctic_crown", "Atlantic", 50),
-        ([-179, 179], [55, 90], -100, ["T"], "arctic_centered", "Pacific", 1),
-        (None, None, None, ["T"], "arctic_crown", "Atlantic", 50),
-        (None, None, None, None, "wrong", "Atlantic", 50),
-        (None, None, None, None, False, "Atlantic", np.nan),
+        (
+            [-90, 20],
+            [20, 60],
+            None,
+            ["T"],
+            50,
+            56,
+            110,
+        ),
+        (
+            [-170, 170],
+            [20, 60],
+            None,
+            ["U", "V"],
+            50,
+            56,
+            359,
+        ),
+        ([-31, -2], [58, 85.2], None, None, 50, 62, 34),
+        ([160, -150], [58, 85.2], None, None, 50, 61, 48),
+        ([60, 130], [58, 85.2], None, None, 50, 62, 68),
+        (None, None, None, ["T"], 50, 314, 359),
+        (None, None, None, None, 50, 314, 359),
     ],
 )
 def test_cutout_faces(
-    od, XRange, YRange, ZRange, varList, transformation, centered, NZ
+    od,
+    XRange,
+    YRange,
+    ZRange,
+    varList,
+    NZ,
+    NY,
+    NX,
 ):
     args = {
         "varList": varList,
         "XRange": XRange,
         "YRange": YRange,
         "ZRange": ZRange,
-        "transformation": transformation,
-        "centered": centered,
     }
-    if transformation not in ["arctic_crown", "arctic_centered"]:
-        with pytest.raises(ValueError):
-            new_od = od.subsample.cutout(**args)
-    else:
-        new_od = od.subsample.cutout(**args)
-        old_dims = od.dataset.dims
-        new_dims = new_od.dataset.dims
-        assert (set(old_dims) - set(new_dims)) == set(["face"])
-        assert new_od._ds.dims["Z"] == NZ
+    new_od = od.subsample.cutout(**args)
+    old_dims = od.dataset.dims
+    new_dims = new_od.dataset.dims
+
+    Yp1 = new_od.dataset["Yp1"]
+    Xp1 = new_od.dataset["Xp1"]
+
+    assert (set(old_dims) - set(new_dims)) == set(["face"])
+    assert new_dims["Z"] == NZ
+    assert new_dims["Y"] == NY
+    assert new_dims["X"] == NX
+    assert new_dims["Y"] == new_dims["Yp1"] - 1
+    assert new_dims["X"] == new_dims["Xp1"] - 1
+
+    assert int(Yp1[-1]) == new_dims["Yp1"] + int(Yp1[0]) - 1
+    assert int(Xp1[-1]) == new_dims["Xp1"] + int(Xp1[0]) - 1
 
 
 # =======
