@@ -2,10 +2,12 @@
 OceanSpy utilities that don't need OceanDataset objects.
 """
 
+import copy as _copy
+
+import numpy as _np
+
 # Required dependencies (private)
 import xarray as _xr
-import numpy as _np
-import copy as _copy
 
 # From oceanspy (private)
 from ._ospy_utils import _check_instance
@@ -16,6 +18,137 @@ try:
 except ImportError:  # pragma: no cover
     pass
 
+try:
+    import numba
+
+    has_numba = True
+except ImportError:
+    has_numba = False
+
+
+def compilable(f):
+    if has_numba:
+        return numba.njit(f)
+    return f
+
+
+def viewer_to_range(p):
+    """
+    Takes the output from the poseidon viewer `p` and returns the coordinate
+    trajectories in X and Y in a way that is compatible with oceanspy.subsample
+    functions.
+    Parameters
+    ----------
+    p: list.
+        data exported from the Poseidon viewer
+    Returns
+    -------
+    lon: list.
+    lat: list.
+    """
+    _check_instance({"p": p}, {"p": ["list"]})
+    _check_instance({"p[0]": p[0]}, {"p[0]": ["dict"]})
+    _check_instance({"type": p[0]["type"]}, {"type": "str"})
+    _check_instance({"coord": p[0]["coordinates"]}, {"coord": "list"})
+
+    if p[0]["type"] in ["Polygon", "LineString", "Point"]:
+        # print messege
+        print("extracting " + p[0]["type"])
+    else:
+        raise TypeError("not a type extracted by poseidon viewer")
+
+    p_type = p[0]["type"]
+
+    if p_type == "Polygon":
+        coords = p[0]["coordinates"][0]
+    elif p_type in ["LineString", "Point"]:
+        coords = p[0]["coordinates"]
+
+    lon = []
+    lat = []
+
+    for i in range(len(coords)):
+        lon.append(coords[i][0])
+        lat.append(coords[i][1])
+
+    return lon, lat
+
+
+def _rel_lon(x, ref_lon):
+    """
+    Change the definition of 0 longitude.
+    Return how much east one need to go from ref_lon to x
+    This function aims to address the confusion caused by
+    the discontinuity in longitude.
+    Parameters
+    ----------
+    x: float, numpy.array, dask.array
+        longitude to convert
+    ref_lon: float
+        a reference longitude. This longitude will become 0deg
+    Returns
+    -------
+    redefined_x: float, numpy.array, dask.array
+        converted longitude
+    """
+    return (x - ref_lon) % 360
+
+
+def _reset_range(x):
+    """Resets the definition of XRange, by default the discontinuity at 180 long.
+    Checks that there is no sign change in x and if there is, the only change that
+    is allowed is when crossing zero. Otherwise resets ref_lon.
+
+    Parameters
+    ----------
+
+    x: numpy.array.
+        array with longitude values.
+
+    Returns
+    -------
+
+    x0, x1: numpy.array
+        endpoints of cutout.
+    redefined_x: numpy.array.
+        converted longitude.
+    """
+
+    ref_lon = 180
+    if x is not None:
+        if (_np.sign(x) == _np.sign(x[0])).all():  # no sign change
+            _ref_lon = ref_lon
+            X0, X1 = _np.min(x), _np.max(x)
+        else:  # change in sign
+            if len(x) == 2:  # list of end points
+                X0, X1 = x
+                if x[0] > x[1]:  # across discontinuity
+                    if abs(x[1] - x[0]) > 300:  # across a discontinuity (Delta X =360)
+                        _ref_lon = x[0] - (x[0] - x[1]) / 3
+                    else:  # XRange decreases, but not necessarity a dicont.
+                        _ref_lon = ref_lon
+                else:
+                    _ref_lon = ref_lon
+            else:  # array of values.
+                _del = abs(x[1:] - x[:-1])  # only works with one crossing
+                if len(_np.where(abs(_del) > 300)[0]) > 0:  # there's discontinuity
+                    ll = _np.where(_del == max(_del))[0][0]
+                    if x[ll] > x[ll + 1]:  # track starts west of jump
+                        X0 = _np.min(x[: ll + 1])
+                        X1 = _np.max(x[ll + 1 :])
+                    else:
+                        X0 = _np.min(x[ll + 1 :])
+                        X1 = _np.max(x[: ll + 1])
+                    _ref_lon = X0 - (X0 - X1) / 3
+                else:  # no discontinuity
+                    X0 = _np.min(x)
+                    X1 = _np.max(x)
+                    _ref_lon = ref_lon
+        x = _np.array([X0, X1])
+    else:
+        _ref_lon = ref_lon
+    return x, _np.round(_ref_lon, 2)
+
 
 def spherical2cartesian(Y, X, R=None):
     """
@@ -161,12 +294,12 @@ def great_circle_path(lat1, lon1, lat2, lon2, delta_km=None, R=None):
         # Define the vector that is normal to
         # both unit_radial_1 & unit_radial_2
         normal_vec = _np.cross(unit_radial_1, unit_radial_2)
-        unit_normal = normal_vec / _np.sqrt(_np.sum(normal_vec ** 2))
+        unit_normal = normal_vec / _np.sqrt(_np.sum(normal_vec**2))
 
         # Define the vector that is tangent to the great circle flight path at
         # (lat1,lon1)
         tangent_1_vec = _np.cross(unit_normal, unit_radial_1)
-        unit_tangent_1 = tangent_1_vec / _np.sqrt(_np.sum(tangent_1_vec ** 2))
+        unit_tangent_1 = tangent_1_vec / _np.sqrt(_np.sum(tangent_1_vec**2))
 
         # Determine the total arc distance from 1 to 2 (radians)
         total_arc_angle_1_to_2 = _np.arccos(unit_radial_1.dot(unit_radial_2))
@@ -546,7 +679,7 @@ def static_pressure(Z):
     """
     # Coefficients to determine static pressure
     c = [0.059808, -0.025, 0.100766, 2.28405e-7]
-    P = c[0] * _np.exp((c[1] * Z) - 1) + c[2] * Z + c[3] * (Z ** 2)
+    P = c[0] * _np.exp((c[1] * Z) - 1) + c[2] * Z + c[3] * (Z**2)
     P = 10 * P  # decibars
     return P
 
@@ -584,3 +717,208 @@ def Coriolis_parameter(Y, omega=7.2921e-5):
     e = 2 * omega * _np.cos(Y_rad)
 
     return f, e
+
+
+def get_maskH(ds, add_Hbdr, XRange, YRange, ref_lon=0):
+    """Define this function to avoid repeated code. First time this runs,
+    the objective is to figure out which faces survive the cutout. This info
+    is then passed, when transforming llc-grids, to llc_rearrange. Second
+    time this code runs, it gets applied on a dataset without faces as a
+    dimension.
+    """
+    if "face" in ds.dims and len(ds.X) == 4320:
+        args = {"Xp1": slice(0, 4320, 10), "Yp1": slice(0, 4320, 10)}
+        ds = _copy.deepcopy(ds.isel(**args))
+
+    maskH = _xr.ones_like(ds["XG"])
+
+    if YRange is not None:
+        # Use arrays
+        YRange = _np.asarray([_np.min(YRange) - add_Hbdr, _np.max(YRange) + add_Hbdr])
+        YRange = YRange.astype(ds["YG"].dtype)
+
+        # Get the closest
+        for i, Y in enumerate(YRange):
+            diff = _np.fabs(ds["YG"] - Y)
+            YRange[i] = ds["YG"].where(diff == diff.min()).min().values
+        maskH = maskH.where(
+            _np.logical_and(ds["YG"] >= YRange[0], ds["YG"] <= YRange[-1]), 0
+        )
+
+    if XRange is not None:
+        # Use arrays
+        XRange = _np.asarray([XRange[0] - add_Hbdr, XRange[-1] + add_Hbdr])
+        XRange = XRange.astype(ds["XG"].dtype)
+
+        # Get the closest
+        for i, X in enumerate(XRange):
+            diff = _np.fabs(ds["XG"] - X)
+            XRange[i] = ds["XG"].where(diff == diff.min()).min().values
+        maskH = maskH.where(
+            _np.logical_and(
+                _rel_lon(ds["XG"], ref_lon) >= _rel_lon(XRange[0], ref_lon),
+                _rel_lon(ds["XG"], ref_lon) <= _rel_lon(XRange[-1], ref_lon),
+            ),
+            0,
+        )
+    # Can't be all zeros
+    if maskH.sum() == 0:
+        raise ValueError("Zero grid points in the horizontal range")
+
+    # Find horizontal indexes
+    maskH = maskH.assign_coords(
+        Yp1=_np.arange(len(maskH["Yp1"])), Xp1=_np.arange(len(maskH["Xp1"]))
+    )
+    dmaskH = maskH.where(maskH, drop=True)
+    return maskH, dmaskH, XRange, YRange
+
+
+@compilable
+def spherical2cartesian_compiled(Y, X, R=6371.0):
+    """
+    Convert spherical coordinates to cartesian.
+
+    Parameters
+    ----------
+    Y: np.array
+        Spherical Y coordinate (latitude)
+    X: np.array
+        Spherical X coordinate (longitude)
+    R: scalar
+        Earth radius in km
+        If None, use geopy default
+    Returns
+    -------
+    x: np.array
+        Cartesian x coordinate
+    y: np.array
+        Cartesian y coordinate
+    z: np.array
+        Cartesian z coordinate
+    """
+
+    # Convert
+    Y_rad = _np.deg2rad(Y)
+    X_rad = _np.deg2rad(X)
+    x = R * _np.cos(Y_rad) * _np.cos(X_rad)
+    y = R * _np.cos(Y_rad) * _np.sin(X_rad)
+    z = R * _np.sin(Y_rad)
+
+    return x, y, z
+
+
+@compilable
+def to_180(x):
+    """
+    convert any longitude scale to [-180,180)
+
+    Parameters
+    ----------
+    x: float ,numpy.array
+        angles in degree
+
+    Returns
+    -------
+    redefined_x: float,numpy.ndarray
+        the same angle coverted to [-180,180)
+    """
+    x = x % 360
+    return x + (-1) * (x // 180) * 360
+
+
+def local_to_latlon(u, v, cs, sn):
+    """
+    convert local vector to north-east
+
+    Parameters
+    ----------
+    u: float, numpy.array
+        the x component of the vector
+    v: float, numpy.array
+        the y component of the vector
+    cs: float, numpy.array
+        the cosine of the angle between grid and compass
+    sn: float, numpy.array
+        the sine of the angle between grid and compass
+
+    Returns
+    -------
+    uu: float, numpy.array
+        the eastward component of the vector
+    vv: float, numpy.array
+        the northward component of the vector
+    """
+    uu = u * cs - v * sn
+    vv = u * sn + v * cs
+    return uu, vv
+
+
+def get_combination(lst, select):
+    """
+    Iteratively find all the combination that
+    has (select) amount of elements
+    and every element belongs to lst
+
+    Parameters
+    ----------
+    lst: list
+        a iterable object to select from
+    select: int
+        the number of objects to select
+
+    Returns
+    -------
+    com_list: list
+        a list of all the possible combinations
+    """
+    # TODO: see if the one in itertools can replace this
+    if select == 1:
+        return [[num] for num in lst]
+    else:
+        the_lst = []
+        for i, num in enumerate(lst):
+            sub_lst = get_combination(lst[i + 1 :], select - 1)
+            for com in sub_lst:
+                com.append(num)
+            #             print(sub_lst)
+            the_lst += sub_lst
+        return the_lst
+
+
+def find_cs_sn(thetaA, phiA, thetaB, phiB):
+    """
+    theta is the angle
+    between the meridian crossing point A
+    and the geodesic connecting A and B
+
+    this function return cos and sin of theta
+
+    Parameters
+    ----------
+    thetaA: float,numpy.array
+        the latitude of the vertex of the angle
+    phiA: float,numpy.array
+        the longitude of the vertex of the angle
+    thetaB: float,numpy.array
+        the latitude of the point B
+    phiB: float,numpy.array
+        the longitude of the point B
+
+    Returns
+    -------
+    CS: float,numpy.array
+        the cosine of the angle
+    SN: float, numpy.array
+        the sine of the angle
+    """
+    # O being north pole
+    AO = _np.pi / 2 - thetaA
+    BO = _np.pi / 2 - thetaB
+    dphi = phiB - phiA
+    # Spherical law of cosine on AOB
+    cos_AB = _np.cos(BO) * _np.cos(AO) + _np.sin(BO) * _np.sin(AO) * _np.cos(dphi)
+    sin_AB = _np.sqrt(1 - cos_AB**2)
+    # spherical law of sine on triangle AOB
+    SN = _np.sin(BO) * _np.sin(dphi) / sin_AB
+    CS = _np.sign(thetaB - thetaA) * _np.sqrt(1 - SN**2)
+    return CS, SN
